function metrics = measureLines04s(edges,parameters)

h = guihandles();
ps = str2double(h.parmPS.String);
if ~isfield(edges,'Average_PSD_LWR')
    Output = edges;
    
    %Le = Output.leadingEdgeProfilesFilled.*ps;
    %Te = Output.trailingEdgeProfilesFilled.*ps;
    Le_f = Output.leadingEdgeProfilesFilledCorrected.*ps;
    Te_f = Output.trailingEdgeProfilesFilledCorrected.*ps;
    LWvar = Output.LWvar*ps;
    %Evaluate average CD
    LW_f = Te_f-Le_f; %line widths
    
    LC_f = (Te_f+Le_f)/2; %line centers
    ALC_f = mean(LC_f); %mean center per line
    ACD_f = mean(LW_f); %Average CD per line
    ALC_f = ALC_f(ACD_f<1.5*mean(ACD_f)); %Discard lines with missing edges
    ACD_f = ACD_f(ACD_f<1.5*mean(ACD_f)); %Discard lines with missing edges
    LinesCenters = ALC_f;
    LinesCD = ACD_f;
    mCD = mean(ACD_f); %Average CD
    stdCD = std(ACD_f); %CD Standard deviation
    
    %Evaluate average LWR PSD
    LW = LWvar; %line widths
    LW = LW-nanmean(LW,1);
    %Evaluate average LER PSD
    Le_f = Le_f-nanmean(Le_f,1); %line widths
    Te_f = Te_f-nanmean(Te_f,1);
    
    F_LW = fft(LW);  
    F_Le = fft(Le_f);
    F_Te = fft(Te_f);
    
    %Calculate the correlation length
    sm = mean(std(LW)); %Average LWR standard deviation
    sml = mean(std(Le_f)); %Average standard deviation Le
    smt = mean(std(Te_f)); %Average standard deviation Te
    
    [H1,r] = hhcf(LW(:,1)-mean(LW(:,1)),ps);
    H1 = zeros(size(H1,1),size(LW,2));
    for k = 1:size(LW,2)
        [H1(:,k),r1] = hhcf(LW(:,k)-mean(LW(:,k)),ps);
    end
    H1 = mean(H1,2); %Average height-height correlation function
    beta0_1 = [sm 20 1 0];
    
        [H2,r] = hhcf(Le_f(:,1)-mean(Le_f(:,1)),ps);
    H2 = zeros(size(H2,1),size(Le_f,2));
    for k = 1:size(Le_f,2)
        [H2(:,k),r2] = hhcf(Le_f(:,k)-mean(Le_f(:,k)),ps);
    end
    H2 = mean(H2,2); %Average height-height correlation function
    beta0_2 = [sml 20 1 0];
    
        [H3,r] = hhcf(Te_f(:,1)-mean(Te_f(:,1)),ps);
    H = zeros(size(H3,1),size(Te_f,2));
    for k = 1:size(Te_f,2)
        [H3(:,k),r3] = hhcf(Te_f(:,k)-mean(Te_f(:,k)),ps);
    end
    H3 = mean(H3,2); %Average height-height correlation function
    beta0_3 = [smt 20 1 0];
    
    try
        beta = nlinfit(r_1',H1,@hhcfmod,beta0_1);
    catch
        beta =beta0_1;
    end
    
    
%     v1 = find(H<1.2642*sm^2,1,'last');
%     v2 = find(H>1.2642*sm^2,1,'first');
%     if isempty(v1)
%         LWRCL = ps;
%     else
%         LWRCL = v1*ps + (1.2642*sm^2-H(v1))*ps*(v2-v1)/(H(v2)-H(v1));
%     end
    HHCorrFunc = H;
    HHCorrFuncFit = hhcfmod(beta,r);
    LWRCorrLength = 2*beta(2);
    
    %PSD_LWR = mF_LW;
    %PSD_LER1 = mF_LEl;
    %PSD_LER2 = mF_LEt;
    %PSD_LER = mF_LE;
    
    LWR3s = 3*mean(std(LW));
    
    %%%%Calculate bias
    
    %Evaluate average LWR
    N = size(LW,1);
    Fs = 1/ps;
    freq = 0:Fs/N:Fs/2-Fs/N;
    freqL = 0:Fs/N:Fs-Fs/N;
    %F_LW = F_LW(1:length(freq),:);
    
    mF_LW = nanmean(abs(F_LW).^2,2); % Average LW PSD (uncut)
    
%     L = length(mF_LW);
%     
%     mF_LW = mF_LW/L;
    
    
    %Estimate max of PSD  form the first FN elements
    %Estimate floor of PSD  form the last LN elements
    %Guess of the correlation length: CL
    %Exclude the first ExN elements from the fit.
    
    FN  = str2double(h.parmFN.String);
    LN  = str2double(h.parmLN.String);
    CF = str2double(h.ed_CF.String);
    ExN = str2double(h.parmExN.String);
    ExNe = str2double(h.parmExNe.String);
    Alpha = str2double(h.ed_alpha.String);
    MFE = str2double(h.ed_MFE.String);
    MI = str2double(h.ed_MIT.String);
    
    %Variables for the fit
    freqx = freq(ExN+1:end-ExNe);
    mF_LWx= mF_LW(1:length(freq));
    mF_LWx= mF_LWx(ExN+1:end-ExNe);
    Lx = length(mF_LWx);
    
    PSDmodel = h.PSDmodel.String{h.PSDmodel.Value};
    
    if strcmp(PSDmodel,'Gaussian')
        model = @Gaussian;
        beta0 = [nanmean(mF_LWx(1:FN))-nanmean(mF_LWx(Lx-LN:Lx)),1/CF,nanmean(mF_LWx(Lx-LN:Lx))];
        f = @(beta,freqx,mF_LWx) sum((Gaussian(beta,freqx)-mF_LWx).^2);
        Options = optimset('MaxFunEvals',MFE,'MaxIter',MI);
        betaf = fminsearch(@(beta) f(beta,freqx,mF_LWx),beta0,Options);
        mF_LW_fit = model(betaf,freq);
        betan = betaf;
        betan(3) = 0;
        mF_LW_fit_unbiased = model(betan,freq);
    elseif strcmp(PSDmodel,'FloatingAlpha')
        model = @FloatingAlpha;
        beta0 = [nanmean(mF_LWx(1:FN))-nanmean(mF_LWx(Lx-LN:Lx)),1/CF,nanmean(mF_LWx(Lx-LN:Lx)),Alpha];
        f = @(beta,freqx,mF_LWx) sum((FloatingAlpha(beta,freqx)-mF_LWx).^2);
        Options = optimset('MaxFunEvals',MFE,'MaxIter',MI);
        betaf = fminsearch(@(beta) f(beta,freqx,mF_LWx),beta0,Options);
        mF_LW_fit = model(betaf,freq);
        betan = betaf;
        betan(3) = 0;
        mF_LW_fit_unbiased = model(betan,freq);
    elseif strcmp(PSDmodel,'NoWhiteNoise')
        model = @Palasantzas1;
        beta0 = [nanmean(mF_LWx(1:FN)), CF, 0,2];
        f = @(beta,freqx,mF_LWx) sum((Palasantzas2(beta,freqx)-mF_LWx).^2);    
        Options = optimset('MaxFunEvals',MFE,'MaxIter',MI);
        betaf = fminsearch(@(beta) f(beta,freqx,mF_LWx),beta0,Options);
        mF_LW_fit = model(betaf,freq);
        betan = betaf;
        betan(3) = 0;
        mF_LW_fit_unbiased = model(betan,freq);
    elseif strcmp(PSDmodel,'Palasantzas1')
        model = @Palasantzas1;
        beta0 = [sqrt((nanmean(mF_LWx(1:FN))-nanmean(mF_LWx(Lx-LN:Lx)))*CF), CF, (nanmean(mF_LWx(Lx-LN:Lx))),Alpha,1];
        f = @(beta,freqx,mF_LWx) sum((Palasantzas1(beta,freqx)-mF_LWx).^2);
        Options = optimset('MaxFunEvals',MFE,'MaxIter',MI);
        betaf = fminsearch(@(beta) f(beta,freqx,mF_LWx),beta0,Options);
        %betaf = beta0;
        mF_LW_fit = model(betaf,freq);
        betan = betaf;
        betan(3) = 0;
        mF_LW_fit_unbiased = model(betan,freq);
    elseif strcmp(PSDmodel,'Palasantzas2')
        model = @Palasantzas2;
        beta0 = [sqrt(nanmean(mF_LWx(1:FN))-nanmean(mF_LWx(Lx-LN:Lx))), CF, (nanmean(mF_LWx(Lx-LN:Lx))),Alpha];
        f = @(beta,freqx,mF_LWx) sum((Palasantzas2(beta,freqx)-mF_LWx).^2);
        Options = optimset('MaxFunEvals',MFE,'MaxIter',MI);
        betaf = fminsearch(@(beta) f(beta,freqx,mF_LWx),beta0,Options);
        mF_LW_fit = model(betaf,freq);
        betan = betaf;
        betan(3) = 0;
        mF_LW_fit_unbiased = model(betan,freq);    
    elseif strcmp(PSDmodel,'Integral')
        model = @Integral;
        %Rv = abs((-N/2:N/(N-1):N/2)*ps);
        edf=[find(freq==freqx(1),1),find(freq==freqx(end),1)];
        beta0 = [sqrt(nanmean(mF_LWx(1:FN))-nanmean(mF_LWx(Lx-LN:Lx))), 1/CF, (nanmean(mF_LWx(Lx-LN:Lx))),Alpha];
        f = @(beta,Rv,edf,mF_LWx) sum((Integral(beta,freqL,edf)-mF_LWx).^2);
        Options = optimset('MaxFunEvals',MFE,'MaxIter',MI);
        betaf = fminsearch(@(beta) f(beta,freqL,edf,mF_LWx),beta0,Options);
        %edf=[find(freq==freqx(1),1)-1,find(freq==freqx(end),1)];
        %disp(beta0)
        %disp(betaf)
        %betaf = beta0;
        
        %mF_LW_fit = model(betaf,freqL,[edf(1),length(freq)]);
        mF_LW_fit = model(betaf,freqL,[1,length(freq)]);
        betan = betaf;
        betan(3) = 0;
        mF_LW_fit_unbiased = model(betan,freqL,[1,length(freq)]);
    end
    
    
    
    %LWR = 3*sqrt(sum(2*mF_LW.^2)/ps);
    
    PSD = mF_LW;
    PSDh = mF_LW(1:length(freq));
    
    PSD_unbiased = (PSDh-betaf(3));
    PSD_unbiased(PSD_unbiased<0)=0;
    
    PSD_LWR_fit = mF_LW_fit;
    
    PSD_LWR_fit_unbiased = mF_LW_fit_unbiased;
    
    %LWR = 3*sqrt(2*ps^2*sum(PSD)/(length(PSD)));
    LWR = 3*sqrt(sum(PSD))/length(PSD);
    LWR_unbiased = 3*sqrt(2*sum(PSD_unbiased))/length(PSD);
    LWR_fit = 3*sqrt(2*sum(PSD_LWR_fit))/length(PSD);
    LWR_fit_unbiased = 3*sqrt(2*sum(PSD_LWR_fit_unbiased))/length(PSD);
    
    
    PSD_LWR_beta = betaf;
    
    
    %%%%
    % imagesc(Output.Arc);axis image
    % hold on
    % for m = 1:size(Le,2)
    %     plot(squeeze(Le(:,m)./ps),1:size(Output.Arc,1),'linewidth',1,'color','r')
    %     plot(squeeze(Te(:,m)./ps),1:size(Output.Arc,1),'linewidth',1,'color','r')
    % end
    % hold off
    % drawnow
    
    metrics.LWprofiles = LW;
    
    metrics.HHCorrFunc = HHCorrFunc;
    metrics.HHCorrFuncFit = HHCorrFuncFit;
    metrics.LWRCorrLength = LWRCorrLength;
    metrics.LinesCD = LinesCD;
    metrics.PSD_LWR_fit = PSD_LWR_fit;
    metrics.PSD_LWR_fit_unbiased = PSD_LWR_fit_unbiased;
    metrics.PSD_LWR_beta = PSD_LWR_beta;
    metrics.LinesCenters = LinesCenters;
    
    metrics.PSD = PSDh;
    metrics.mCD = mCD;
    metrics.stdCD = stdCD;
    metrics.LWR3s = LWR3s;
    metrics.LWR = LWR;
    metrics.LWR_unbiased = LWR_unbiased;
    metrics.LWR_fit = LWR_fit;
    metrics.LWR_fit_unbiased = LWR_fit_unbiased;
    metrics.freq = freq;
    metrics.beta0 = beta0;
    metrics.beta = betaf;
    metrics.r = r;
else   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%AVERAGE IMAGES CASE - Not fully implemented yet
    mF_LW = edges.Average_PSD_LWR;
    N = size(mF_LW,1);
    Fs = 1/ps;
    freq = 0:Fs/N:Fs/2-Fs/N;
    L = length(mF_LW);
    
    FN  = parameters.FN;
    LN  = parameters.LN;
    CF = parameters.CF;
    ExN = parameters.ExN;
    Alpha = parameters.Alpha;
    
    
    
    freqx = freq(ExN:end);
    mF_LWx= PSD(ExN:end);
    
    PSDmodel = parameters.PSDmodel;
    
    if strcmp(PSDmodel,'Gaussian')
        model = @LWRModel4;
        beta0 = [nanmean(mF_LW(1:FN))-nanmean(mF_LW(L-LN:L)),1/CF,nanmean(mF_LW(L-LN:L))];
        f = @(beta,freqx) sum((LWRModel4(beta,freqx)-mF_LWx).^2);
        Options = optimset('MaxFunEvals',parameters.MFE,'MaxIter',parameters.MI);
        betaf = fminsearch(@(beta) f(beta,freqx,mF_LWx),beta0,Options);
        mF_LW_fit = model(betaf,freq);
        betan = betaf;
        betan(3) = 0;
        mF_LW_fit_unbiased = model(betan,freq);
    elseif strcmp(PSDmodel,'FloatingAlpha')
        model = @LWRModel3;
        beta0 = [nanmean(mF_LW(1:FN))-nanmean(mF_LW(L-LN:L)),1/CF,nanmean(mF_LW(L-LN:L)),Alpha];
        f = @(beta,freqx) sum((LWRModel3(beta,freqx)-mF_LWx).^2);
        Options = optimset('MaxFunEvals',parameters.MFE,'MaxIter',parameters.MI);
        betaf = fminsearch(@(beta) f(beta,freqx,mF_LWx),beta0,Options);
        mF_LW_fit = model(betaf,freq);
        betan = betaf;
        betan(3) = 0;
        mF_LW_fit_unbiased = model(betan,freq);
    elseif strcmp(PSDmodel,'NoWhiteNoise')
        model = @LWRModel5;
        beta0 = [nanmean(mF_LW(1:FN))-nanmean(mF_LW(L-LN:L)),1/CF,0];
        f = @(beta,freqx) sum((LWRModel5(beta,freqx)-mF_LWx).^2);
        Options = optimset('MaxFunEvals',parameters.MFE,'MaxIter',parameters.MI);
        betaf = fminsearch(@(beta) f(beta,freqx,mF_LWx),beta0,Options);
        mF_LW_fit = model(betaf,freq);
        betan = betaf;
        betan(3) = 0;
        mF_LW_fit_unbiased = model(betan,freq);
    elseif strcmp(PSDmodel,'Palasantzas')
        model = @LWRModel6;
        beta0 = [nanmean(mF_LW(1:FN))-nanmean(mF_LW(L-LN:L)), CF, sqrt(nanmean(mF_LW(L-LN:L))),2];
        f = @(beta,freqx,mF_LWx) sum((LWRModel6(beta,freqx)-mF_LWx).^2);
        Options = optimset('MaxFunEvals',parameters.MFE,'MaxIter',parameters.MI);
        betaf = fminsearch(@(beta) f(beta,freqx,mF_LWx),beta0,Options);
        mF_LW_fit = model(betaf,freq);
        betan = betaf;
        betan(3) = 0;
        mF_LW_fit_unbiased = model(betan,freq);
    end
    
    
    %LWR = 3*sqrt(sum(2*mF_LW.^2)/ps);
    LWR = 3*sqrt(sum(mF_LW));
    
    PSD = mF_LW*(size(LW,1));
    %PSD_unbiased = (mF_LW-betaf(3))*(size(LW,1));
    PSD_unbiased = (mF_LW-betaf(3));
    PSD_unbiased(PSD_unbiased<0)=0;
    
    LWR_unbiased = 3*sqrt(sum(PSD_unbiased));
    LWR_fit = 3*sqrt(sum(mF_LW_fit));
    LWR_fit_unbiased = 3*sqrt(sum(mF_LW_fit_unbiased));
    
    
    PSD_LWR_fit = abs(mF_LW_fit);
    PSD_LWR_fit_unbiased = abs(mF_LW_fit_unbiased)*(size(LW,1));
    PSD_LWR_beta = betaf;
    
    
end